@@ -1,18 +1,17 @@
 try:
     from pytrends.request import TrendReq
-    from twitterscraper import query_tweets
 except:
     pass
 
 import asyncio
-from datetime import date, datetime, timedelta
 import time
+from datetime import date, datetime, timedelta
 
 import aiohttp
-from bs4 import BeautifulSoup
 import numpy as np
 import pandas as pd
 import requests
+from bs4 import BeautifulSoup
 
 
 class Karpet:
@@ -273,83 +272,6 @@ class Karpet:
 
         return df.sort_values("date").reset_index(drop=True)
 
-    # def fetch_tweets(self, kw_list, lang, limit=None):
-    #     """
-    #     Scrapes Twitter without any limits and  returns dataframe with the
-    #     following structure
-
-    #     * fullname
-    #     * id
-    #     * likes
-    #     * replies
-    #     * retweets
-    #     * text
-    #     * timestamp
-    #     * url
-    #     * user
-    #     * date
-    #     * has_link
-
-    #     :param list kw_list: List of keywords to search for. Will be joined with "OR" operator.
-    #     :param str lang: Language of tweets to search in.
-    #     :param int limit: Limit search results. Might get really big and slow so this should help.
-    #     :return: Pandas dataframe with all search results (tweets).
-    #     :rtype: pd.DataFrame
-    #     """
-
-    #     def process_tweets(tweets):
-    #         """
-    #         Cleans up tweets and returns dataframe with the
-    #         following structure
-
-    #         * fullname
-    #         * id
-    #         * likes
-    #         * replies
-    #         * retweets
-    #         * text
-    #         * timestamp
-    #         * url
-    #         * user
-    #         * date
-    #         * has_link
-
-    #         :param list tweets: List of dicts of tweets data.
-    #         :return: Returns dataframe with all the scraped tweets (no index).
-    #         :rtype: pd.DataFrame
-    #         """
-
-    #         # 1. Clean up.
-    #         data = []
-
-    #         for t in tweets:
-    #             d = t.__dict__
-    #             del d["html"]
-    #             data.append(d)
-
-    #         # 2. Create dataframe
-    #         df = pd.DataFrame(data)
-    #         # import pdb
-
-    #         # pdb.set_trace()
-    #         df["date"] = df["timestamp"].dt.date
-    #         df["has_link"] = df["text"].apply(
-    #             lambda text: "http://" in text or "https://" in text
-    #         )
-
-    #         return df
-
-    #     try:
-    #         _ = query_tweets
-    #     except NameError:
-    #         raise Exception("Twitter extension is not installed - see README file.")
-
-    #     tweets = query_tweets(
-    #         query=" OR ".join(kw_list), begindate=self.start, lang=lang, limit=limit
-    #     )
-
-    #     return process_tweets(tweets)
-
     def fetch_news(self, symbol, limit=10):
         """
         Fetches news of the given symbol. Each news contains
@@ -513,6 +435,7 @@ class Karpet:
         - name
         - current_price
         - market_cap
+        - rank
         - reddit_average_posts_48h
         - reddit_average_comments_48h
         - reddit_subscribers
@@ -527,6 +450,8 @@ class Karpet:
         - year_low
         - year_high
         - yoy_change
+        - price_change_24
+        - price_change_24_percents
 
         :param str symbol: Coin symbol - i.e. BTC, ETH, ...
         :param str id: Coin ID (baed on coingecko.com).
@@ -537,9 +462,7 @@ class Karpet:
 
         id = self._get_coin_id_from_params(symbol, id)
 
-        data = self._get_json(
-            f"https://api.coingecko.com/api/v3/coins/{id}/history?date={date.today().strftime('%d-%m-%Y')}"
-        )
+        data = self._get_json(f"https://api.coingecko.com/api/v3/coins/{id}")
         data_chart = self._get_json(
             f"https://api.coingecko.com/api/v3/coins/{id}/market_chart?vs_currency=usd&days=365"
         )
@@ -556,6 +479,7 @@ class Karpet:
             "name": data["name"],
             "current_price": data["market_data"]["current_price"]["usd"],
             "market_cap": data["market_data"]["market_cap"]["usd"],
+            "rank": data["market_data"]["market_cap_rank"],
             "reddit_average_posts_48h": data["community_data"][
                 "reddit_average_comments_48h"
             ],
@@ -564,7 +488,7 @@ class Karpet:
             ],
             "reddit_subscribers": data["community_data"]["reddit_subscribers"],
             "reddit_accounts_active_48h": float(
-                data["community_data"]["reddit_accounts_active_48h"]
+                data["community_data"]["reddit_accounts_active_48h"] or 0
             ),
             "forks": data["developer_data"]["forks"],
             "stars": data["developer_data"]["stars"],
@@ -578,8 +502,13 @@ class Karpet:
             "year_high": sorted_chart_by_price[-1][1],
             "yoy_change": 100
             * (sorted_chart_by_date[-1][1] / sorted_chart_by_date[0][1] - 1),
+            "price_change_24": data["market_data"]["price_change_24h"],
+            "price_change_24_percents": data["market_data"][
+                "price_change_percentage_24h"
+            ],
         }
 
+        # Calculate open issues.
         if (
             data["developer_data"]["total_issues"]
             and data["developer_data"]["closed_issues"]
@@ -624,43 +553,46 @@ class Karpet:
             :param object news: News object.
             """
 
-            async with session.get(news["url"]) as response:
+            try:
+                async with session.get(news["url"]) as response:
 
-                html = await response.text()
-                dom = BeautifulSoup(html, features="lxml")
+                    html = await response.text()
+                    dom = BeautifulSoup(html, features="lxml")
 
-                # Title.
-                try:
-                    news["title"] = dom.find("meta", {"property": "og:title"})[
-                        "content"
-                    ]
-                except:
-                    news["title"] = None
+                    # Title.
+                    try:
+                        news["title"] = dom.find("meta", {"property": "og:title"})[
+                            "content"
+                        ]
+                    except:
+                        news["title"] = None
 
-                # Date.
-                try:
-                    d = dom.find("meta", {"property": "article:published_time"})[
-                        "content"
-                    ]
-                    news["date"] = datetime.strptime(d, "%Y-%m-%dT%H:%M:%S%z")
-                except:
-                    news["date"] = None
+                    # Date.
+                    try:
+                        d = dom.find("meta", {"property": "article:published_time"})[
+                            "content"
+                        ]
+                        news["date"] = datetime.strptime(d, "%Y-%m-%dT%H:%M:%S%z")
+                    except:
+                        news["date"] = None
 
-                # Image.
-                try:
-                    news["image"] = dom.find("meta", {"property": "og:image"})[
-                        "content"
-                    ]
-                except:
-                    news["image"] = None
+                    # Image.
+                    try:
+                        news["image"] = dom.find("meta", {"property": "og:image"})[
+                            "content"
+                        ]
+                    except:
+                        news["image"] = None
 
-                # Description.
-                try:
-                    news["description"] = dom.find(
-                        "meta", {"property": "og:description"}
-                    )["content"]
-                except:
-                    news["description"] = None
+                    # Description.
+                    try:
+                        news["description"] = dom.find(
+                            "meta", {"property": "og:description"}
+                        )["content"]
+                    except:
+                        news["description"] = None
+            except (aiohttp.ClientResponseError, UnicodeDecodeError):
+                pass
 
         async with aiohttp.ClientSession() as session:
             await fetch_all(session, news)
@@ -679,7 +611,7 @@ class Karpet:
         filtered_news = []
 
         for n in news:
-            if not n["date"]:
+            if "date" not in n or not n["date"]:
                 continue
 
             filtered_news.append(n)
